"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PgConditionCustomFieldsPlugin = void 0;
require("graphile-config");
const graphile_export_1 = require("graphile-export");
const behavior_js_1 = require("../behavior.js");
const version_js_1 = require("../version.js");
exports.PgConditionCustomFieldsPlugin = {
    name: "PgConditionCustomFieldsPlugin",
    description: "Add GraphQL conditions based on 'filterable' PostgreSQL functions",
    version: version_js_1.version,
    schema: {
        hooks: {
            GraphQLInputObjectType_fields(fields, build, context) {
                const { inflection, sql } = build;
                const { scope: { isPgCondition, pgCodec }, fieldWithHooks, } = context;
                if (!isPgCondition ||
                    !pgCodec ||
                    !pgCodec.attributes ||
                    pgCodec.isAnonymous) {
                    return fields;
                }
                const functionSources = Object.values(build.input.pgRegistry.pgResources).filter((resource) => {
                    if (resource.codec.attributes)
                        return false;
                    if (resource.codec.arrayOfCodec)
                        return false;
                    if (resource.codec.rangeOfCodec)
                        return false;
                    const parameters = resource.parameters;
                    if (!parameters || parameters.length < 1)
                        return false;
                    if (parameters.some((p, i) => i > 0 && p.required))
                        return false;
                    if (parameters[0].codec !== pgCodec)
                        return false;
                    if (!resource.isUnique)
                        return false;
                    const behavior = (0, behavior_js_1.getBehavior)([
                        resource.codec.extensions,
                        resource.extensions,
                    ]);
                    return build.behavior.matches(behavior, "proc:filterBy", "-proc:filterBy");
                });
                return build.extend(fields, functionSources.reduce((memo, rawPgFieldSource) => {
                    const pgFieldSource = rawPgFieldSource;
                    const fieldName = inflection.computedAttributeField({
                        resource: pgFieldSource,
                    });
                    const type = build.getGraphQLTypeByPgCodec(pgFieldSource.codec, "input");
                    if (!type)
                        return memo;
                    memo = build.extend(memo, {
                        [fieldName]: fieldWithHooks({
                            fieldName,
                            fieldBehaviorScope: "proc:filterBy",
                            isPgConnectionConditionInputField: true,
                            pgFieldSource,
                        }, {
                            description: build.wrapDescription(`Checks for equality with the object’s \`${fieldName}\` field.`, "field"),
                            type,
                            applyPlan: (0, graphile_export_1.EXPORTABLE)((pgFieldSource, sql) => function plan($condition, val) {
                                if (typeof pgFieldSource.from !== "function") {
                                    throw new Error("Invalid computed attribute 'from'");
                                }
                                const expression = sql `${pgFieldSource.from({
                                    placeholder: $condition.alias,
                                })}`;
                                if (val.getRaw().evalIs(null)) {
                                    $condition.where(sql `${expression} is null`);
                                }
                                else {
                                    $condition.where(sql `${expression} = ${$condition.placeholder(val.get(), pgFieldSource.codec)}`);
                                }
                            }, [pgFieldSource, sql]),
                        }),
                    }, `Adding computed attribute condition argument for ${pgCodec.name}`);
                    return memo;
                }, Object.create(null)), `Adding computed attribute filterable functions to condition for '${pgCodec.name}'`);
            },
        },
    },
};
//# sourceMappingURL=PgConditionCustomFieldsPlugin.js.map